'------------------------------------------------------------------------------
' This program is Copyright (c) 2005 Microsoft Corporation.
'
' All rights reserved.
'
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
' ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
' THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
' PARTICULAR PURPOSE.
'
' IN NO EVENT SHALL MICROSOFT AND/OR ITS RESPECTIVE SUPPLIERS BE
' LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
' DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
' WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
' ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
' OF THIS CODE OR INFORMATION.
'------------------------------------------------------------------------------
'
'     ISA Server 2004 (Objets COM FPC)
'
'     Connectivity Verifiers configuration update - v1.2
'
'     Arguments : /c : Name of Connectivity Verifier to touch
'                 /all : To update all CVs (This is the default)
'                 /r : Name of report file (Default = "None", no file)
'                 /t : Threshold (in milliseconds)
'                 /e : Enable CV
'                 /d : Disable CV
'                 /rr : RefreshRate, time in seconds between two tests
'                      (warning : common value for all cvs, needs FW restart)
'                 /restart : Restart Firewall service (usefull with /rr)
'
'                 /h or /? or /help : Affiche l'aide
'
'     Microsoft Consulting Services - jcornier@microsoft.com - 2005/01
'------------------------------------------------------------------------------

Option Explicit

Dim objFPCRoot
Dim iErrorNumber
Dim strErrorDescription, strLogString

Dim bDisplay, bAllCV, bEnable, bDisable, bRR, bRestart
Dim arrCV()
Dim iDistinctCV

' Create root object for ISA Server FPC (ISA)
Set objFPCRoot = CreateObject("FPC.Root")

bDisplay = TRUE

Main

'------------------------------------------------------------------------------
'    Main Procedure
'------------------------------------------------------------------------------
Sub Main()
    Dim objArgs
    Dim iArgs, iNbArgs
    Dim bCVok
    Dim strCurrDir, strFileName, strQuestion
    Dim iLoop
    Dim StartProcess
    Dim oFile
    Dim iCV
    Dim strCVName
    Dim lThreshold
    Dim iRefreshRate
    Dim strThreshold, strRefreshRate, strLocalServerName

    'Default values
    bAllCV = FALSE
    bEnable = FALSE
    bDisable = FALSE
    bRR = FALSE
    bRestart = FALSE
    strFileName = "None"
    lThreshold = 0
    iRefreshRate = 0

    Set objArgs = WScript.Arguments
    iNbArgs = (objArgs.Count - 1)

    '--------------------------------------------------------------------------
    ' Set values passed in command line mode
    '--------------------------------------------------------------------------
    If iNbArgs >= 0 Then

        bDisplay = FALSE
        bCVok = FALSE

        For iArgs = 0 to iNbArgs
            Select Case LCase(objArgs(iArgs))
                Case "/?", "-h", "/help", "-help" ' Help
                    DisplayHelp
                Case "/c", "-c" ' Cv to find
                    If (iArgs + 1) <= iNbArgs Then
                        strCVName = objArgs(iArgs + 1)
                        bCVok = TRUE
                    Else
                        DisplayHelp
                    End If
                Case "/all", "-all" ' All CVs
                    bAllCV = TRUE
                    bCVok = TRUE
                    strCvName = "All"
                Case "/r", "-r" ' Report File
                    If (iArgs + 1) <= iNbArgs Then
                        strFileName = objArgs(iArgs + 1)
                    Else
                        DisplayHelp
                    End If
                Case "/t", "-t" ' Threshold
                    If (iArgs + 1) <= iNbArgs Then
                        lThreshold = CLng(objArgs(iArgs + 1))
                    Else
                        DisplayHelp
                    End If
                Case "/rr", "-rr" ' RefreshRate
                    If (iArgs + 1) <= iNbArgs Then
                        bRR = TRUE
                        iRefreshRate = CLng(objArgs(iArgs + 1))
                    Else
                        DisplayHelp
                    End If
                Case "/e", "-e" ' Enable
                    bEnable = TRUE
                Case "/d", "-d" 'Disable
                    bDisable = TRUE
                Case "/restart", "-restart" ' Restart Firewall Service
                    bRestart = TRUE
            End Select
        Next

        If bCVok = FALSE Then
            DisplayHelp
        End If

    Else
    '--------------------------------------------------------------------------
    ' If no arguments then show message boxes
    '--------------------------------------------------------------------------
        ' Connectivity Verifier to update
        strCVName = InputBox ("Name of Connectivity Verifier to update:" & _
            VbCrLf & VbCrLf & _
            "(This can run in command line mode too - " & _
            "/h give a list of arguments available)", _
            "Connectivity Verifier Name", "All")
        If strCVName = "" Then
            ProcessCanceled "Script canceled by user"
        End If
        If LCase(strCVName) = "all" Then
            bAllCV = TRUE
        End If

        ' Report file name
        strFileName = strDate() & "_CV.txt"
        strFileName = InputBox ("Name of report file:", _
            "Report file", _
            strFileName)
        If strFileName = "" Then
            ProcessCanceled "Script canceled by user, " & _
                VbCrLf & "or no file name was entered."
        End If

        ' Threshold
        strThreshold = InputBox ("Enter Threshold value " & _
            "(in milliseconds)" & VbCrLf & _
            "Enter 0 to keep current value", _
            "Threshold", "0")
        If strThreshold = "" Then
            ProcessCanceled "Script canceled by user"
        End If
        lThreshold = CLng(strThreshold)

        ' RefreshRate
        strRefreshRate = InputBox ("Enter RefreshRate value " & _
            "(in seconds)" & VbCrLf & _
            "Enter 0 to keep current value", _
            "RefreshRate", "0")
        If strRefreshRate = "" Then
            ProcessCanceled "Script canceled by user"
        End If
        iRefreshRate = CLng(strRefreshRate)
        If iRefreshRate > 0 Then
            bRR = TRUE
        End If

        ' Disable CV
        If 6 = MsgBox("Update Verifier to disable state ?", _
            vbQuestion + vbYesNo + _
            vbApplicationModal, "Connectivity Verifier") Then
            bDisable = TRUE
        End If

        ' Enable CV
        If 6 = MsgBox("Update Verifier to enable state ?", _
            vbQuestion + vbYesNo + _
            vbApplicationModal, "Connectivity Verifier") Then
            bEnable = TRUE
        End If

        ' Restart Firewall service
        If 6 = MsgBox("Restart Firewall service ?", vbQuestion + _
            vbYesNo + vbApplicationModal, "Firewall service") Then
            bRestart = TRUE
        End If

    End If

    '--------------------------------------------------------------------------
    ' Start of automatic process
    '--------------------------------------------------------------------------
    StartProcess = Date()

    ' Check values
    '--------------------------------------------------------------------------
    If bEnable and bDisable Then
        bEnable = FALSE
        bDisable = FALSE
    End If
    
    If bRR Then
        If iRefreshRate < 15 Then
            iRefreshRate = 15
        End If
        If iRefreshRate > 1440 Then
            iRefreshRate = 1440
        End If
    End If

    ' Set current path for files (strCurrDir)
    '--------------------------------------------------------------------------
    Set oFile = WScript.CreateObject("Scripting.FileSystemObject")
    strCurrDir = oFile.GetAbsolutePathName(".")
    Set oFile = Nothing

    ' Set report file (erase if exist)
    '--------------------------------------------------------------------------
    If strFileName = "" Then
        strFileName = "None"
    Else
        strFileName = strCurrDir & "\" & strFileName
    End If
    call LogAction(strFileName, "ISA Server 2004 - Connectivity Verifiers Update", TRUE)
    call LogAction(strFileName, "Connectivity Verifier: " & strCVName, FALSE)
    call LogAction(strFileName, VbCrLf & Date() & " " & Time() & VbCrLf, FALSE)

    '__________________________________________________________________________
    '______________________________ Core ______________________________________

    '- Step 1 -----------------------------------------------------------------
    ' List all connectivity verifiers if option /all is specified
    ' else load the array with only the name of wonnectivity verifier
    ' given as argument
    '--------------------------------------------------------------------------
    If bAllCV Then
        call ListAllCv(objFPCRoot, strFileName)
    Else
        ReDim arrCV(2)

        iDistinctCV = 1
        arrCV(iDistinctCV) = strCVName
    End If


    '- Step 2 -----------------------------------------------------------------
    ' Update connectivity verifiers
    '--------------------------------------------------------------------------
    For iCV = 1 to iDistinctCV
        strCVName = arrCV(iCV)
        call UpdateCV(objFPCRoot, strFileName, strCVName, lThreshold, 0)
    Next

    ' Set RefreshRate
    If bRR Then
        call UpdateCV(objFPCRoot, strFileName, "", 0, iRefreshRate)
    End If

    '- Step 3 -----------------------------------------------------------------
    ' Restart Firewall Service
    '--------------------------------------------------------------------------
    If bRestart Then
        strLocalServerName = RetrieveCurrentComputerName()
        call RestartFirewallService(objFPCRoot, strFileName, strLocalServerName)
    End If

    '_____________________________ End Core ___________________________________
    '__________________________________________________________________________

    ' End of report file
    strLogString = VbCrLf & Date() & " " & Time()
    LogAction strFileName, strLogString, FALSE

    If bDisplay Then
        WScript.Echo "Fin"
    End If
End Sub

'------------------------------------------------------------------------------
'                         End of main code
'                   Start of prodecure and functions
'------------------------------------------------------------------------------

'----------------------------------------------------------------
'    Purpose : Display a message
'             then stop script on user request
'    Arguments : Message to display into the box
'----------------------------------------------------------------
Sub ProcessCanceled(StrQuestion)

    MsgBox strQuestion,vbExclamation + vbOkOnly + vbApplicationModal + 256, _
        "End of Script"
    WScript.Quit(1)
End Sub

'----------------------------------------------------------------
'    Purpose : Give a string representing the current date
'    Return : YearMonthDay, format "yyyymmdd"
'----------------------------------------------------------------
Function strDate()
    Dim strToday, strMonth, strDay, strYear

    strToday = Date()
    strMonth = Month(strToday)
    strDay = Day(strToday)
    If Len(strMonth) < 2 Then strMonth = "0" & strMonth End If
    If Len(strDay) < 2 Then strDay = "0" & strDay End If
    strDate = Year(strToday) & strMonth & strDay
End Function

'----------------------------------------------------------------
'    Purpose : display help
'----------------------------------------------------------------
Sub DisplayHelp()

    WScript.Echo "ISA Server 2004 - Connectivity Verifiers update" & _
    VbCrLf & _
    VbCrLf & "     Usage:" & _
    VbCrLf & "          /c : Name of Connectivity Verifier to update" & _
    VbCrLf & _
    VbCrLf & "         Optional :" & _
    VbCrLf & "          /all: Update all existing connectivity verifiers" & _
    VbCrLf & "                 (This is the default)" & _
    VbCrLf & "          /r: Report file name (Default = None, no file)" & _
    VbCrLf & "          /t: Threshold in milliseconds" & _
    VbCrLf & "          /d: Set CV to enable state" & _
    VbCrLf & "          /e: Set CV to disable state" & _
    VbCrLf & "          /rr: Refresh Rate in seconds (between two tests" & _
    VbCrLf & "                (Common value for all cvs, need FW restart)" & _
    VbCrLf & "          /restart: Restart Firewall service (usefull with /rr)" & _
    VbCrLf & _
    VbCrLf & "          /h or /? or /help: Display help"

    WScript.Quit(-1)
End Sub

'----------------------------------------------------------------
'    Purpose : Append a line into a texte file
'             Create file if needed
'    Arguments : Name of file
'            line to write
'            create or append
'
'    No operation if file name is set to = "None"
'----------------------------------------------------------------
Sub LogAction(strFileLog, strStringToLog, bCreate)
    Dim oFileLog
    Dim TxtLog

    ' Direct return if no file is given (None value)
    If LCase(Right(strFileLog, 4)) = "none" Then
        Exit Sub
    End If

    ' Open file
    Set oFileLog = WScript.CreateObject("Scripting.FileSystemObject")

    If bCreate = TRUE Then
        Set TxtLog = oFileLog.OpenTextFile (strFilelog, 2, True)
    Else
        Set TxtLog = oFileLog.OpenTextFile (strFileLog, 8, True)
    End If

    ' Update file
    TxtLog.Writeline strStringToLog

    ' Close file
    TxtLog.close
    Set oFileLog = Nothing
 End Sub

'------------------------------------------------------------
'    Purpose : List all Connectivity Verifiers
'    Arguments : FPC (ISA) Root Object
'            Report file name
'
'    Global : CVs counter
'             Array of CVs
'             Error Code - 0 if ok
'             Error Description
'------------------------------------------------------------
Sub ListAllCv(ObjFPC, strFileLog)
    Dim array       ' An FPCArray object
    Dim Verifiers   ' An FPCConnectivityVerifiers collection
    Dim Verifier    ' An FPCConnectivityVerifier object

    'On Error Resume Next

    ' Load collection of Verifiers
    Set array = ObjFPC.GetContainingArray
    Set Verifiers = array.ConnectivityVerifiers
    
    ReDim arrCV(Verifiers.count)

    For Each verifier in verifiers
        iDistinctCV = iDistinctCV + 1
        arrCV(iDistinctCV) = Verifier.Name
    Next

    If Err.Number <> 0 Then
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & "CV List" & _
            VbCrLf & "    " & strErrorDescription
        call LogAction(strFileLog, strLogString, FALSE)
        If bDisplay Then
            WScript.Echo strLogString
        End If
    End If

    ' Display nb of Verifiers found if script is running in interactive mode
    If bDisplay Then
        WScript.Echo "Nb of Verifiers: " & iDistinctCV
    End If

    Set Verifier = Nothing
    Set Verifiers = Nothing
End Sub

'------------------------------------------------------------
'    Purpose : Mise  jour d'un connecivity verifier
'    Arguments : FPC (ISA) Root Object
'            Report file name
'            Name of CV to update
'            Threshold
'            RefreshRate
'
'    Global : Error Code - 0 if ok
'             Error Description
'------------------------------------------------------------
Sub UpdateCv(ObjFPC, strFileLog, strVerifierName, lThreshold, iRefreshRate)
    Dim array       ' An FPCArray object
    Dim Verifiers   ' An FPCConnectivityVerifiers collection
    Dim Verifier    ' An FPCConnectivityVerifier object

    ' Constants
    const fpcGroupDNS = 2
    const fpcGroupWeb = 4
    const fpcHttpRequest = 0
    const fpcPingRequest = 1

    'On Error Resume Next

    ' Load collection of Verifiers
    Set array = ObjFPC.GetContainingArray
    Set Verifiers = array.ConnectivityVerifiers
    
    ' Update RefreshRate for all verifiers
    If iRefreshRate <> 0 Then
        strLogString = VbCrLf & "Refresh Rate: " & iRefreshRate
        Verifiers.RefreshRate = iRefreshRate
        Verifiers.Save

        If Err.Number <> 0 Then
            strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & "Refresh Rate: " & iRefreshRate & _
            VbCrLf & "    " & strErrorDescriptionVbCrLf
        End If

        call LogAction(strFileLog, strLogString, FALSE)
        Set Verifier = Nothing
        Set Verifiers = Nothing

        Exit Sub
    End If

    ' Load current verifier
    Set Verifier = verifiers.Item(strVerifierName)
    
    If Err.Number <> 0 Then
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & "Verifier: " & strVerifierName & _
            VbCrLf & "    " & strErrorDescription
        call LogAction(strFileLog, strLogString, FALSE)
        If bDisplay Then
            WScript.Echo strLogString
        End If
    Else
        strLogString = strVerifierName
        
        ' Update Threshold
        If lThreshold <> 0 then
            Verifier.Threshold = lThreshold
            strLogString = strLogString & "; Threshold: " & lThreshold
        End If
        
        ' Enable or disable Verifier (or nothing)
        If bEnable Then
            Verifier.Enabled = TRUE
            strLogString = strLogString & "; Enable"
        End If
        If bDisable Then
            Verifier.Enabled = FALSE
            strLogString = strLogString & "; Disable"
        End If
        
        ' Save new parameters
        Verifier.Save
        
        ' Records actions in report file
        call LogAction(strFileLog, strLogString, FALSE)
    End If
    
    Set Verifier = Nothing
    Set Verifiers = Nothing
End Sub

'----------------------------------------------
' Purpose : Load local server name
' Return : Name of local server
'----------------------------------------------
Function RetrieveCurrentComputerName()
    Dim strComputer
    Dim objWMIService
    Dim colSettings
    Dim objComputer

    strComputer = "."
    Set objWMIService = GetObject("winmgmts:" _
        & "{impersonationLevel=impersonate}!\\" _
        & strComputer & "\root\cimv2")
    Set colSettings = objWMIService.ExecQuery _
        ("Select * from Win32_ComputerSystem")
    For Each objComputer in colSettings
        RetrieveCurrentComputerName = objComputer.Name
    Next
    set objWMIService = Nothing
End Function


'------------------------------------------------------------
'    Purpose : Restart Firewall service
'    Arguments : FPC (ISA) Root Object
'            Report file name
'
'             Error Code - 0 if ok
'             Error Description
'------------------------------------------------------------
Sub RestartFirewallService(ObjFPC, strFileLog, strServerName)
    Dim array
    Dim Servers
    Dim Server
    Dim iCount
    Dim bFwStatus

    'On Error Resume Next
    
    bFwStatus = FALSE    ' FALSE : FW Service not available or stopped
    
    ' Load collection of serveurs
    Set array = ObjFPC.GetContainingArray
    Set Servers = array.Servers

    ' Loop to find local server then stop and restart Firewall service
    For Each Server in Servers
        If LCase(Server.Name) = LCase(strServerName) Then
        
            '-------------------------------------------------------------
            ' Stop Firewall Service
            '-------------------------------------------------------------
            ' Stop Service
            Server.StopFirewallService TRUE
            Wscript.Sleep 6000

            ' Record state in report file
            strLogString = VbCrLf & Now() & " - Firewall Service Stopped"
            call LogAction(strFileLog, strLogString, FALSE)

            '-------------------------------------------------------------
            ' Start Firewall Service, wait for start
            ' iCount purpose is to avoid infinite loop
            '-------------------------------------------------------------
            iCount = 0
            Do While Not bFwStatus And iCount < 1000
                iCount = iCount + 1
                ' Start Service
                Server.StartFirewallService FALSE
                ' Pause
                WScript.Sleep 1000
                Server.Refresh
                bFwStatus = Server.FirewallServiceStatus
            Loop

            ' Record state in report file
            strLogString = Now() & " - Firewall Service Restarted"
            call LogAction(strFileLog, strLogString, FALSE)

            Exit For
        End If
    Next
    
    If Err.Number <> 0 Then
        iErrorNumber = Err.Number
        strErrorDescription = Err.Description

        strLogString = "    Error nb : " & iErrorNumber & _
            VbCrLf & "CV List" & _
            VbCrLf & "    " & strErrorDescription
        call LogAction(strFileLog, strLogString, FALSE)
        If bDisplay Then
            WScript.Echo strLogString
        End If
    End If

    If bDisplay Then
        WScript.Echo "Firewall service is running"
    End If
    
    Set Server = Nothing
    Set Servers = Nothing
End Sub

